local gr = love.graphics
local canvas = {gr.newCanvas(800, 600), gr.newCanvas(800, 600)}

local pv = [[extern float power; varying vec2 bc[13];
			vec4 position(mat4 transform_projection, vec4 vertex_position)
			{
				bc[ 0] = VaryingTexCoord.st + vec2(-0.028 * power, 0.0);
				bc[ 1] = VaryingTexCoord.st + vec2(-0.024 * power, 0.0);
				bc[ 2] = VaryingTexCoord.st + vec2(-0.020 * power, 0.0);
				bc[ 3] = VaryingTexCoord.st + vec2(-0.016 * power, 0.0);
				bc[ 4] = VaryingTexCoord.st + vec2(-0.012 * power, 0.0);
				bc[ 5] = VaryingTexCoord.st + vec2(-0.008 * power, 0.0);
				bc[ 6] = VaryingTexCoord.st + vec2(-0.004 * power, 0.0);
				bc[ 7] = VaryingTexCoord.st + vec2( 0.004 * power, 0.0);
				bc[ 8] = VaryingTexCoord.st + vec2( 0.008 * power, 0.0);
				bc[ 9] = VaryingTexCoord.st + vec2( 0.012 * power, 0.0);
				bc[10] = VaryingTexCoord.st + vec2( 0.016 * power, 0.0);
				bc[11] = VaryingTexCoord.st + vec2( 0.020 * power, 0.0);
				bc[12] = VaryingTexCoord.st + vec2( 0.024 * power, 0.0);

				return transform_projection * vertex_position;
			}
	]]

local ph = [[extern float power; varying vec2 bc[13];
			vec4 position(mat4 transform_projection, vec4 vertex_position)
			{
				bc[ 0] = VaryingTexCoord.st + vec2(0.0, -0.028 * power);
				bc[ 1] = VaryingTexCoord.st + vec2(0.0, -0.024 * power);
				bc[ 2] = VaryingTexCoord.st + vec2(0.0, -0.020 * power);
				bc[ 3] = VaryingTexCoord.st + vec2(0.0, -0.016 * power);
				bc[ 4] = VaryingTexCoord.st + vec2(0.0, -0.012 * power);
				bc[ 5] = VaryingTexCoord.st + vec2(0.0, -0.008 * power);
				bc[ 6] = VaryingTexCoord.st + vec2(0.0, -0.004 * power);
				bc[ 7] = VaryingTexCoord.st + vec2(0.0,  0.004 * power);
				bc[ 8] = VaryingTexCoord.st + vec2(0.0,  0.008 * power);
				bc[ 9] = VaryingTexCoord.st + vec2(0.0,  0.012 * power);
				bc[10] = VaryingTexCoord.st + vec2(0.0,  0.016 * power);
				bc[11] = VaryingTexCoord.st + vec2(0.0,  0.020 * power);
				bc[12] = VaryingTexCoord.st + vec2(0.0,  0.024 * power);

				return transform_projection * vertex_position;
			}
	]]

local blur = [[extern float power; varying vec2 bc[13];
			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords)
			{
				vec4 texcolor = vec4(0.0);
				texcolor += Texel(texture, bc[ 0]) * 0.0044299121055113265;
				texcolor += Texel(texture, bc[ 1]) * 0.00895781211794;
				texcolor += Texel(texture, bc[ 2]) * 0.0215963866053;
				texcolor += Texel(texture, bc[ 3]) * 0.0443683338718;
				texcolor += Texel(texture, bc[ 4]) * 0.0776744219933;
				texcolor += Texel(texture, bc[ 5]) * 0.115876621105;
				texcolor += Texel(texture, bc[ 6]) * 0.147308056121;
				texcolor += Texel(texture, texture_coords) * 0.159576912161;
				texcolor += Texel(texture, bc[ 7]) * 0.147308056121;
				texcolor += Texel(texture, bc[ 8]) * 0.115876621105;
				texcolor += Texel(texture, bc[ 9]) * 0.0776744219933;
				texcolor += Texel(texture, bc[10]) * 0.0443683338718;
				texcolor += Texel(texture, bc[11]) * 0.0215963866053;
				texcolor += Texel(texture, bc[12]) * 0.00895781211794;

				return texcolor;// * color;
			}
	]]

local hblur = gr.newShader(blur, ph)
local vblur = gr.newShader(blur, pv)

function love.load()
	love.window.setTitle('Blur Test')
	gr.setBackgroundColor(40, 40, 150, 255)

	power = 1
	hblur:send("power", power)
	vblur:send("power", power)

	segs = 8

	r = love.math.random(255)
	g = love.math.random(255)
	b = love.math.random(255)
	a = love.math.random(255)
	r2 = love.math.random(255)
	g2 = love.math.random(255)
	b2 = love.math.random(255)
	a2 = love.math.random(255)

	text = {
		x = 200,
		y = 350,
		w = 400,
		h = 200,
		'Press "Insert" to create random mirrored shape',
		'Press "Up" to increase blur power',
		'Press "Down" to decrease blur power',
		'Press "Page Up" to increase number of markers',
		'Press "Page Down" to decrease number of markers',
		'Press "Home / End" to change shape colors',
		'Press "Escape" to quit'
		}
	coords = gr.roundrect('points', text.x, text.y, text.w, text.h, 20)
end

function love.update(dt)
	if love.keyboard.isDown("up") then
		power = power + 0.01
		hblur:send("power", power)
		vblur:send("power", power)
	end
	if love.keyboard.isDown("down") then
		power = power - 0.01
		hblur:send("power", power)
		vblur:send("power", power)
	end
end

function love.draw()
	canvas[1]:clear()
	canvas[2]:clear()

	gr.setColor(r, g, b, 255)	-- draw figure to screen
	gr.polygon("fill", coords)

	gr.setCanvas(canvas[1])		-- draw figure to canvas[1]
		gr.setColor(r2, g2, b2, 255)
		gr.polygon("fill", coords)

	gr.setCanvas(canvas[2])		-- draw canvas[1] to canvas[2] with hblur
		gr.setShader(hblur)
		gr.draw(canvas[1])

	gr.setCanvas()

	gr.setStencil(stencil)		-- draw canvas[2] to screen with vblur (confined to shape)
		gr.setShader(vblur)
			gr.draw(canvas[2])
		gr.setShader()
	gr.setStencil()

	if tbl then
		gr.setColor(255, 0, 0, 255)
		for i = 1, #tbl do
			gr.push()
			gr.translate(tbl[i].x, tbl[i].y)
			gr.rotate(tbl[i].angle)
			gr.circle('fill',0,0,10,3)
			gr.pop()
		end
	end
	gr.setColor(0,0,0,255)
	for i=1,#text do
		gr.print(text[i],text.x+30,text.y+10+20*i)
		end
end

function love.keypressed(key, isRepeat)
	if key == "escape" then
		love.event.quit()
	end
	if key == "pageup" then
		segs = segs + 1
		even(coords, segs)
	end
	if key == "pagedown" then
		segs = segs - 1
		if segs == 0 then segs = 1 end
		even(coords, segs)
	end
	if key == "home" then
		r = love.math.random(255)
		g = love.math.random(255)
		b = love.math.random(255)
		a = love.math.random(255)
	end
	if key == "end" then
		r2 = love.math.random(255)
		g2 = love.math.random(255)
		b2 = love.math.random(255)
		a2 = love.math.random(255)
	end
	if key == "insert" then
		createMirrorBezier()
	end
end

function stencil()	-- crops off blur that extends bejond shape
	gr.polygon("fill", coords)
end

function getAngle(x1, y1, x2, y2)
	return math.atan2(x2 - x1, y2 - y1)
end

function even(points, num)
	tbl = {}

	lengths = {}
	len = 0
	gr.setColor(0, 0, 0, 255)
	j = 1
	for i = 1, #points - 2, 2 do
		lengths[j] = {}
		lengths[j].coord = i
		lengths[j].length = getLength(points[i], points[i + 1], points[i + 2], points[i + 3])
		len = len + lengths[j].length
		j = j + 1
	end

	length = len / num

	tbl[1] = {}
	tbl[1].x = points[1]
	tbl[1].y = points[2]
	tbl[1].angle = getAngle(points[1], points[2], points[3], points[4])
	curlen = 0
	tb = 2
	for i = 1, #lengths - 4 do
		if length > curlen + lengths[i].length then
			curlen = curlen + lengths[i].length
		else
			tbl[tb] = {}
			tbl[tb].x, tbl[tb].y = getPoint(length - curlen, points[lengths[i].coord], points[lengths[i].coord + 1], points[lengths[i].coord + 2], points[lengths[i].coord + 3])
			tbl[tb].angle = getAngle(points[lengths[i + 4].coord], points[lengths[i].coord + 5], points[lengths[i].coord + 6], points[lengths[i].coord + 7])
			curlen = length - curlen
			tb = tb + 1
		end
	end
end

function createMirrorBezier()
	t = {}
	for i = 1, 4 do
		t[i] = {}
		if i == 1 then nextx = 0 else nextx = t[i - 1].x end
		t[i].x = love.math.random(nextx, 399)
		if i == 1 then nexty = 0 else nexty = t[i - 1].y end
		t[i].y = love.math.random(nexty, 299)
	end

	t[5] = {}
	t[6] = {}
	t[7] = {}
	t[8] = {}
	t[5].x = 799 - t[4].x
	t[5].y = t[4].y
	t[6].x = 799 - t[3].x
	t[6].y = t[3].y
	t[7].x = 799 - t[2].x
	t[7].y = t[2].y
	t[8].x = 799 - t[1].x
	t[8].y = t[1].y

	curve = love.math.newBezierCurve(399, 20, t[1].x, t[1].y, t[2].x, t[2].y, t[3].x, t[3].y, t[4].x, t[4].y, t[5].x, t[5].y, t[6].x, t[6].y, t[7].x, t[7].y, t[8].x, t[8].y, 399, 20)
	coords = curve:render(8)

	even(coords, segs)
end

function getLength(x1, y1, x2, y2)
	return math.sqrt((x1 - x2) ^ 2 + (y1 - y2) ^ 2)
end

function getPoint(n, x1, y1, x2, y2)
	d = math.sqrt((x1 - x2) ^ 2 + (y1 - y2) ^ 2)
	r = n / d
	return r * x2 + (1 - r) * x1, r * y2 + (1 - r) * y1
end

function gr.roundrect(mode, x, y, width, height, rx, ry)
	local points = {}
	local rx = rx or 10
	local ry = ry or rx
	local tI, hP = table.insert, .5*math.pi
	if rx > width*.5 then rx = width*.5 end
	if ry > height*.5 then ry = height*.5 end
	local precision = (rx + ry) * .1
	local X1, Y1, X2, Y2 = x + rx, y + ry, x + width - rx, y + height - ry
	local sin, cos = math.sin, math.cos
	for i = 0, precision do
		local a = (i/precision-1)*hP
		tI(points, X2 + rx*cos(a))
		tI(points, Y1 + ry*sin(a))
	end
	for i = 0, precision do
		local a = (i/precision)*hP
		tI(points, X2 + rx*cos(a))
		tI(points, Y2 + ry*sin(a))
	end
	for i = 0, precision do
		local a = (i/precision+1)*hP
		tI(points, X1 + rx*cos(a))
		tI(points, Y2 + ry*sin(a))
	end
	for i = 0, precision do
		local a = (i/precision+2)*hP
		tI(points, X1 + rx*cos(a))
		tI(points, Y1 + ry*sin(a))
	end
	if mode == 'points' then return points end
	gr.polygon( mode, unpack( points ) )
end
